import { ConvertedRequestParam } from '../types';

/**
 * Parse string dot notation to object
 * @property - target property for parsing
 * @value - target property value
 */
const parseDotNotation = (property: string, value: any): Record<string, any> | string => {
  if (property.includes('.')) {
    const [field, ...rest]: string[] = property.split('.');
    const least: string = rest.join('.');
    const res: Record<string, any> = { [field]: least };
    res[field] = parseDotNotation(least, value);
    return res;
  }
  return { [property]: value };
};

/**
 * Parse dont notation value to object and replace it in parent object
 * @param key - Key with dot notation
 * @param target - Target object
 */
export const replaceDotNotationField = (key: string, target: ConvertedRequestParam): string => {
  const hierarchy = key.split('.');

  if (hierarchy.length <= 1) return key;

  const resultObject: Record<string, any> = parseDotNotation(key, target[key]) as Record<
    string,
    any
  >;

  delete target[key];
  const [newKey] = hierarchy;

  target[newKey] = resultObject[newKey];

  return newKey;
};
