import { RMQ_TRANSPORT_DTO_TYPE } from '../common/constants';
import { TargetObject, ValidationDto } from '../common/types';
import { INVALID_DTO_IS_ADDED } from '../common/util/messages';

/**
 * Decorator that specify custom dto for method validation
 * @param validationDto - Dto class, should contain "Dto" part in class name.
 */
export const RmqTransportDtoType =
  (validationDto: ValidationDto): ParameterDecorator =>
  (target: TargetObject, propertyKey: keyof TargetObject | undefined) => {
    if (
      !(validationDto && propertyKey && validationDto.name && validationDto.name.includes('Dto'))
    ) {
      throw new Error(INVALID_DTO_IS_ADDED(propertyKey, validationDto?.name));
    }

    Reflect.defineMetadata(RMQ_TRANSPORT_DTO_TYPE, validationDto, target, propertyKey);
  };
