import { ValidateIf } from 'class-validator';
import { VALIDATE_IF_RELATED_ONLY } from '../common/constants';
import { TargetObject } from '../common/types';
import { ValidateIfRelatedOnlyOptions } from '../interfaces/validate-if-related-only-options.interface';

/**
 * Decorator that specify dto property to participate in validation process for specific groups
 * only in case if parent dto is nested
 * @param options - Decorator options object
 */
export const ValidateIfRelatedOnly =
  (options: ValidateIfRelatedOnlyOptions = { groups: [] }): PropertyDecorator =>
  (target: TargetObject, propertyKey: keyof TargetObject) => {
    Reflect.defineMetadata(VALIDATE_IF_RELATED_ONLY, options, target, propertyKey);
    ValidateIf((object) => object.$isNestedDto, options)(target, propertyKey);
  };
