import { QueryFilterOperators } from '../common/constants';

export interface QueryFilter {
  /**
   * Field name
   */
  field: string;
  /**
   * Logical operator
   */
  operator: QueryFilterOperators;
  /**
   * Field value
   */
  value?: any;
}

type QuerySortOperator = 'ASC' | 'DESC';

export interface QueryJoin {
  /**
   * Field name
   */
  field: string;
  /**
   * Selected fields array
   */
  select?: string[];
}

export interface QuerySort {
  /**
   * Field name
   */
  field: string;
  /**
   * Order ASC / DESC
   */
  order: QuerySortOperator;
}

type SPrimitivesVal = string | number | boolean;
type SFiledValues = SPrimitivesVal | Array<SPrimitivesVal>;

type SFieldOperator = {
  $eq?: SFiledValues;
  $ne?: SFiledValues;
  $gt?: SFiledValues;
  $lt?: SFiledValues;
  $gte?: SFiledValues;
  $lte?: SFiledValues;
  $starts?: SFiledValues;
  $ends?: SFiledValues;
  $cont?: SFiledValues;
  $excl?: SFiledValues;
  $in?: SFiledValues;
  $notin?: SFiledValues;
  $between?: SFiledValues;
  $isnull?: SFiledValues;
  $notnull?: SFiledValues;
  $eqL?: SFiledValues;
  $neL?: SFiledValues;
  $startsL?: SFiledValues;
  $endsL?: SFiledValues;
  $contL?: SFiledValues;
  $exclL?: SFiledValues;
  $inL?: SFiledValues;
  $notinL?: SFiledValues;
  $or?: SFieldOperator;
  $and?: never;
};

export type SField = SPrimitivesVal | SFieldOperator;

export type SCondition = {
  /**
   * Field search query
   */
  [key: string]: SField | SCondition[] | undefined;
  /**
   * "Or" query info
   */
  $or?: SCondition[];
  /**
   * "And" query info
   */
  $and?: SCondition[];
};

export interface ParsedRequestParams {
  /**
   * Fields array
   */
  fields: string[];
  /**
   * Params filter queries array
   */
  paramsFilter: QueryFilter[];
  /**
   * Auth persist object
   */
  authPersist?: Record<string, any>;
  /**
   * Search conditions object
   */
  search: SCondition;
  /**
   * Filter queries array
   */
  filter: QueryFilter[];
  /**
   * "Or" queries array
   */
  or: QueryFilter[];
  /**
   * Join info objects array
   */
  join: QueryJoin[];
  /**
   * Sort info objects array
   */
  sort: QuerySort[];
  /**
   * Number of records to limit
   */
  limit?: number;
  /**
   * Number of records to skip
   */
  offset?: number;
  /**
   * Page number
   */
  page?: number;
  /**
   * Cache status
   */
  cache?: number;
}
