interface ValidationArguments {
  /**
   * Validating value.
   */
  value: any;
  /**
   * Constraints set by this validation type.
   */
  constraints: any[];
  /**
   * Name of the target that is being validated.
   */
  targetName: string;
  /**
   * Object that is being validated.
   */
  // eslint-disable-next-line @typescript-eslint/ban-types
  object: Object;
  /**
   * Name of the object's property being validated.
   */
  property: string;
}

export interface ValidationMetadata {
  type: string;
  /**
   * Target class to which this validation is applied.
   */
  // eslint-disable-next-line @typescript-eslint/ban-types
  target: Function | string;
  /**
   * Property of the object to be validated.
   */
  propertyName: string;
  /**
   * Constraint class that performs validation. Used only for custom validations.
   */
  // eslint-disable-next-line @typescript-eslint/ban-types
  constraintCls: Function;
  /**
   * Array of constraints of this validation.
   */
  constraints: any[];
  /**
   * Validation message to be shown in the case of error.
   */
  message: string | ((args: ValidationArguments) => string);
  /**
   * Validation groups used for this validation.
   */
  groups: string[];
  /**
   * Indicates if validation must be performed always, no matter of validation groups used.
   */
  always?: boolean;
  /**
   * Specifies if validated value is an array and each of its item must be validated.
   */
  each: boolean;
  context?: any;
  /**
   * Extra options specific to validation type.
   */
  validationTypeOptions: any;
}
