import { MicroserviceValidation } from '../../src/decorators/microservice-validation.decorator';
import { BaseDto } from '../dto/base.dto';
import { TargetObject } from '../../src/common/types';
import { RmqTransportDtoType } from '../../src/decorators/rmq-transport-dto-type.decorator';
import { EnabledNestedRelationsDto } from '../dto/enabled-nested-relations.dto';
import { DisabledNestedRelationsDto } from '../dto/disabled-nested-relations.dto';
import { ParsedRequestParams } from '../../src/interfaces/parsed-request-params.interface';

@MicroserviceValidation(BaseDto)
export class TestController {
  // For checking crud method
  createOne(data: { dto: TargetObject }): TargetObject {
    return data.dto;
  }

  // For checking bulk method validation
  createMany(data: {
    dto: {
      bulk: TargetObject[];
    };
  }): TargetObject {
    return data.dto;
  }

  // For checking crud method with custom DTO passed
  updateOne(
    @RmqTransportDtoType(EnabledNestedRelationsDto) data: { dto: TargetObject },
  ): TargetObject {
    return data.dto;
  }

  // For checking request validation
  getOne(data: { req: TargetObject }): ParsedRequestParams {
    return data.req.parsed;
  }

  // For checking request validation if only request is added as argument
  getMany(req: { parsed: ParsedRequestParams; options: Record<string, any> }): ParsedRequestParams {
    return req.parsed;
  }

  // For checking nested relations

  withRelations(
    @RmqTransportDtoType(EnabledNestedRelationsDto) data: { dto: TargetObject },
  ): TargetObject {
    return data.dto;
  }

  requestWithRelations(
    @RmqTransportDtoType(EnabledNestedRelationsDto) data: { req: TargetObject },
  ): ParsedRequestParams {
    return data.req.parsed;
  }
  // For checking nested relations disabling

  withoutRelations(
    @RmqTransportDtoType(DisabledNestedRelationsDto) data: { dto: TargetObject },
  ): TargetObject {
    return data.dto;
  }

  requestWithoutRelations(
    @RmqTransportDtoType(DisabledNestedRelationsDto) data: { req: TargetObject },
  ): ParsedRequestParams {
    return data.req.parsed;
  }
}
