import { IsBoolean, IsOptional, IsString, ValidateNested } from 'class-validator';
import { CrudRequestTypes, RelatedDto } from '../../src';
import { NestedDto } from './nested.dto';

export class BaseDto {
  @RelatedDto()
  @ValidateNested({
    context: NestedDto,
    groups: [CrudRequestTypes.CREATE, CrudRequestTypes.UPDATE, CrudRequestTypes.READ],
  })
  nestedField?: NestedDto;

  @RelatedDto()
  @ValidateNested({
    context: NestedDto,
    groups: [CrudRequestTypes.CREATE, CrudRequestTypes.UPDATE, CrudRequestTypes.READ],
    each: true,
  })
  nestedArray?: NestedDto[];

  @IsString({ groups: [CrudRequestTypes.CREATE, CrudRequestTypes.UPDATE, CrudRequestTypes.READ] })
  requiredField?: string;

  @IsString({ groups: [CrudRequestTypes.CREATE, CrudRequestTypes.UPDATE, CrudRequestTypes.READ] })
  @IsOptional({ groups: [CrudRequestTypes.CREATE, CrudRequestTypes.UPDATE, CrudRequestTypes.READ] })
  optionalField?: string;

  @IsBoolean({ groups: [CrudRequestTypes.CREATE, CrudRequestTypes.UPDATE, CrudRequestTypes.READ] })
  isBaseDto?: boolean;

  @IsBoolean({ groups: [CrudRequestTypes.DELETE] })
  groupsCheck?: boolean;

  invisibleField?: string;
}
