import { TestController } from './controllers/ test.controller';
import {
  CUSTOM_DTO_DECORATOR_IS_REQUIRED,
  INVALID_DTO_IS_ADDED,
  NO_BASE_NO_CUSTOM_DTO,
} from '../src/common/util/messages';
import { NoBaseWithRmqController } from './controllers/no-base-with-rmq.controller';

describe('initialization', () => {
  it('should create instance successfully in case of correct decorators set', () => {
    const testController = new TestController();
    expect(testController).toBeInstanceOf(TestController);
  });

  it('should return error in case if custom method has no custom dto added', () => {
    let err: any;
    try {
      // eslint-disable-next-line global-require
      require('./controllers/custom-without-dto.controller');
    } catch (e) {
      err = e;
    }
    expect(err).toBeInstanceOf(Error);
    expect(err?.message).toEqual(CUSTOM_DTO_DECORATOR_IS_REQUIRED('custom'));
  });

  it('should return error in case if custom method has invalid dto added to RMQTransportDto decorator', () => {
    let err: any;
    try {
      // eslint-disable-next-line global-require
      require('./controllers/custom-with-wrong-dto.controller');
    } catch (e) {
      err = e;
    }
    expect(err).toBeInstanceOf(Error);
    expect(err?.message).toEqual(INVALID_DTO_IS_ADDED('custom', 'WrongName'));
  });

  it('should return error in case if method has no dto added and base validation dto is not added to controller', () => {
    let err: any;
    try {
      // eslint-disable-next-line global-require
      require('./controllers/no-base-without-rmq.controller');
    } catch (e) {
      err = e;
    }
    expect(err).toBeInstanceOf(Error);
    expect(err?.message).toEqual(NO_BASE_NO_CUSTOM_DTO('createOne'));
  });

  it(
    'should create instance successfully in case of correct if base dto is not added ' +
      'but method has custom dto validation',
    () => {
      const testController = new NoBaseWithRmqController();
      expect(testController).toBeInstanceOf(NoBaseWithRmqController);
    },
  );
});
